#version 400 core

uniform sampler2D textureUnit;
uniform sampler2DShadow shadow_map; // shadow map is just a texture

in vec3 normal;
in vec3 position;
in vec4 sm_coord;
in vec3 color;

uniform vec3 ambient;
uniform vec3 light;
uniform vec3 intensity;
uniform vec3 viewing;

uniform float diffuse;
uniform float specular;
uniform float shininess;


out vec4 out_color; // final fragment color


//void main(void) {
//	// perform perspective division
//	vec3 tex_coords = SM_tex_coord.xyz/SM_tex_coord.w;
//
//	// read depth value from shadow map
//	float depth = texture(shadow_map, tex_coords.xy).r;
//	//float depth = texture(texUnit, tex_coords.xy).r;

//	// perform depth comparison
//	float inShadow = (depth < tex_coords.z) ? 1.0 : 0.0;
//
//	// darken fragment color if in shadow...
//	if(inShadow == 1){
//		out_color = vec4(0.5, 0.5, 0.5, 0.1);
//	}else{
//		out_color = vec4(1, 1, 1, 0);
//	}
//	
//}


//subroutine void RenderPassType();
//subroutine uniform RenderPassType RenderPass;

//subroutine(RenderPassType)
void shadeWithShadow()
{
	vec3 L = normalize(light);
	vec3 N = normalize(normal);
	vec3 R = normalize(-reflect(L,N));
	vec3 V = normalize(viewing);
	vec3 diffAndSpec = diffuse * intensity * max(dot(N, L), 0.0) * color + specular * intensity * pow(max(dot(R, V), 0.0), shininess);

	// shadow map lookup
	float shadow = textureProj(shadow_map, sm_coord);
	vec4 dings = texture2D(textureUnit, vec2(5, 5));
	// if in shadow, use only ambient
	/*if(shadow >= 0.9){
		out_color += vec4(diffAndSpec*shadow + (ambient*color) , 0.5);
	}*/
	if (shadow == 1) {
		out_color = vec4(1, 1, 1, 1);
	} else {
		out_color = vec4(ambient.x, ambient.y, ambient.z, 1);
	}
}

//subroutine(RenderPassType)
//void recordDepth()
//{
//	// do nothing, depth will be written automatically
//}

void main(void)
{
	// will call either shadeWithShadow or recordDepth
	//RenderPass();
	shadeWithShadow();
}